@@ -6,9 +6,9 @@
 mod sparse_array_usize;
 
 use super::entry::Entry;
+use crate::List;
 use crate::list;
 use crate::utils::DefaultBuildHasher;
-use crate::List;
 use alloc::vec::Vec;
 use archery::{ArcTK, RcK, SharedPointer, SharedPointerKind};
 use core::borrow::Borrow;
@@ -213,7 +213,6 @@ mod node_utils {
     use super::HashValue;
     use core::hash::BuildHasher;
     use core::hash::Hash;
-    use core::hash::Hasher;
     use core::mem::size_of_val;
 
     /// Returns the index of the array for the given hash on depth `depth`.
@@ -237,11 +236,7 @@ mod node_utils {
     }
 
     pub fn hash<T: ?Sized + Hash, H: BuildHasher>(v: &T, hasher_builder: &H) -> HashValue {
-        let mut hasher = hasher_builder.build_hasher();
-
-        v.hash(&mut hasher);
-
-        hasher.finish()
+        hasher_builder.hash_one(v)
     }
 }
 
@@ -421,7 +416,7 @@ where
 
                                 self.compress();
                             }
-                        };
+                        }
 
                         removed
                     }
@@ -634,7 +629,7 @@ where
                                 *b = Bucket::Single(entry);
                             }
                             _ => (),
-                        };
+                        }
 
                         *bucket = Some(b);
 
@@ -878,6 +873,7 @@ where
         self.size() == 0
     }
 
+    #[allow(clippy::iter_without_into_iter)]
     pub fn iter(&self) -> Iter<'_, K, V, P> {
         self.iter_ptr().map(|e| (&e.key, &e.value))
     }
@@ -917,7 +913,7 @@ where
     }
 }
 
-impl<'a, K, Q: ?Sized, V, P, H: BuildHasher> Index<&'a Q> for HashTrieMap<K, V, P, H>
+impl<K, Q: ?Sized, V, P, H: BuildHasher> Index<&Q> for HashTrieMap<K, V, P, H>
 where
     K: Eq + Hash + Borrow<Q>,
     Q: Hash + Eq,
@@ -971,7 +967,7 @@ where
             return true;
         }
         self.size() == other.size()
-            && self.iter().all(|(key, value)| other.get(key).map_or(false, |v| *value == *v))
+            && self.iter().all(|(key, value)| other.get(key).is_some_and(|v| *value == *v))
     }
 }
 
@@ -1105,7 +1101,7 @@ mod iter_utils {
     }
 }
 
-impl<'a, K, V, P> IterPtr<'a, K, V, P>
+impl<K, V, P> IterPtr<'_, K, V, P>
 where
     K: Eq + Hash,
     P: SharedPointerKind,
@@ -1153,7 +1149,7 @@ where
     }
 }
 
-impl<'a, K: Eq + Hash, V, P> ExactSizeIterator for IterPtr<'a, K, V, P> where P: SharedPointerKind {}
+impl<K: Eq + Hash, V, P> ExactSizeIterator for IterPtr<'_, K, V, P> where P: SharedPointerKind {}
 
 #[cfg(feature = "serde")]
 pub mod serde {
