@@ -3,6 +3,10 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/.
  */
 
+#![allow(clippy::cast_possible_truncation)]
+#![allow(clippy::cast_possible_wrap)]
+#![allow(clippy::cast_sign_loss)]
+
 use super::*;
 use alloc::vec::Vec;
 use pretty_assertions::assert_eq;
@@ -58,8 +62,8 @@ where
         };
 
         self_ok
-            && self.left.as_ref().map_or(true, |l| l.red_nodes_have_black_children())
-            && self.right.as_ref().map_or(true, |r| r.red_nodes_have_black_children())
+            && self.left.as_ref().is_none_or(|l| l.red_nodes_have_black_children())
+            && self.right.as_ref().is_none_or(|r| r.red_nodes_have_black_children())
     }
 
     fn has_binary_search_property(&self) -> bool
@@ -70,15 +74,15 @@ where
             node: &Node<K, V, P>,
             last: &mut Option<K>,
         ) -> bool {
-            let ok_left = node.left.as_ref().map_or(true, |l| go(l, last));
+            let ok_left = node.left.as_ref().is_none_or(|l| go(l, last));
 
             let new_last = node.entry.key.clone();
 
-            let ok_node = last.as_ref().map_or(true, |l| *l < new_last);
+            let ok_node = last.as_ref().is_none_or(|l| *l < new_last);
 
             *last = Some(new_last);
 
-            let ok_right = node.right.as_ref().map_or(true, |r| go(r, last));
+            let ok_right = node.right.as_ref().is_none_or(|r| go(r, last));
 
             ok_left && ok_node && ok_right
         }
@@ -106,16 +110,16 @@ where
     K: Ord + Clone,
 {
     fn check_consistent(&self) -> Result<(), InvariantViolation> {
-        if !self.root.as_ref().map_or(true, |r| r.has_binary_search_property()) {
-            Result::Err(InvariantViolation::BinarySearch)
-        } else if !self.root.as_ref().map_or(true, |r| r.red_nodes_have_black_children()) {
-            Result::Err(InvariantViolation::RedNodeBlackChildren)
-        } else if !self.root.as_ref().map_or(true, |r| r.is_black_height_balanced()) {
-            Result::Err(InvariantViolation::BlackHeightBalanced)
-        } else if !self.root.as_ref().map_or(true, |r| r.color == Color::Black) {
-            Result::Err(InvariantViolation::BlackRoot)
+        if !self.root.as_ref().is_none_or(|r| r.has_binary_search_property()) {
+            Err(InvariantViolation::BinarySearch)
+        } else if !self.root.as_ref().is_none_or(|r| r.red_nodes_have_black_children()) {
+            Err(InvariantViolation::RedNodeBlackChildren)
+        } else if !self.root.as_ref().is_none_or(|r| r.is_black_height_balanced()) {
+            Err(InvariantViolation::BlackHeightBalanced)
+        } else if !self.root.as_ref().is_none_or(|r| r.color == Color::Black) {
+            Err(InvariantViolation::BlackRoot)
         } else if self.root.as_ref().map_or(0, |r| r.count()) != self.size() {
-            Result::Err(InvariantViolation::SizeConsistency)
+            Err(InvariantViolation::SizeConsistency)
         } else {
             Ok(())
         }
@@ -1029,9 +1033,9 @@ mod internal {
 
     #[test]
     fn test_insert() {
+        use rand::SeedableRng;
         use rand::rngs::StdRng;
         use rand::seq::SliceRandom;
-        use rand::SeedableRng;
 
         let limit = 25_000;
         let seed: [u8; 32] = [
@@ -1069,8 +1073,12 @@ mod internal {
             assert!(!map.contains_key(v));
 
             if let Err(error) = map.check_consistent() {
-                panic!("Consistency error in red-black tree ({:?}).  Insertions: {:?}.  Removals: {:?}",
-                       error, &values_insert, &values_remove[0..=i]);
+                panic!(
+                    "Consistency error in red-black tree ({:?}).  Insertions: {:?}.  Removals: {:?}",
+                    error,
+                    &values_insert,
+                    &values_remove[0..=i]
+                );
             }
         }
     }
@@ -1085,9 +1093,9 @@ mod internal {
 
     #[test]
     fn test_remove() {
+        use rand::SeedableRng;
         use rand::rngs::StdRng;
         use rand::seq::SliceRandom;
-        use rand::SeedableRng;
 
         let limit = 25_000;
         let seed: [u8; 32] = [
@@ -1135,6 +1143,8 @@ mod iter {
         assert_eq!(iter_utils::lg_floor(17), 4);
     }
 
+    #[allow(clippy::explicit_iter_loop)]
+    #[allow(clippy::never_loop)]
     #[test]
     fn test_iter_empty() {
         let map: RedBlackTreeMap<i32, i32> = RedBlackTreeMap::new();
@@ -1144,6 +1154,7 @@ mod iter {
         }
     }
 
+    #[allow(clippy::never_loop)]
     #[test]
     fn test_iter_empty_backwards() {
         let map: RedBlackTreeMap<i32, i32> = RedBlackTreeMap::new();
@@ -1153,6 +1164,7 @@ mod iter {
         }
     }
 
+    #[allow(clippy::explicit_iter_loop)]
     #[test]
     fn test_iter_big_map() {
         let limit = 512;
@@ -1746,7 +1758,7 @@ fn test_partial_ord() {
     let map_1_prime = rbt_map!["a" => 0xa];
     let map_2 = rbt_map!["b" => 0xb];
     let map_3 = rbt_map![0 => 0.0];
-    let map_4 = rbt_map![0 => core::f32::NAN];
+    let map_4 = rbt_map![0 => f32::NAN];
 
     assert_eq!(map_1.partial_cmp(&map_1_prime), Some(Ordering::Equal));
     assert_eq!(map_1.partial_cmp(&map_2), Some(Ordering::Less));
@@ -1825,10 +1837,9 @@ fn test_clone() {
 #[cfg(feature = "serde")]
 #[test]
 fn test_serde() {
-    use bincode::{deserialize, serialize};
     let map: RedBlackTreeMap<i32, i32> = rbt_map![5 => 6, 7 => 8, 9 => 10, 11 => 12];
-    let encoded = serialize(&map).unwrap();
-    let decoded: RedBlackTreeMap<i32, i32> = deserialize(&encoded).unwrap();
+    let encoded = serde_json::to_string(&map).unwrap();
+    let decoded: RedBlackTreeMap<i32, i32> = serde_json::from_str(&encoded).unwrap();
 
     assert_eq!(map, decoded);
 }
