@@ -3,6 +3,10 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/.
  */
 
+#![allow(clippy::cast_possible_truncation)]
+#![allow(clippy::cast_possible_wrap)]
+#![allow(clippy::cast_sign_loss)]
+
 use super::*;
 use alloc::string::String;
 use pretty_assertions::assert_eq;
@@ -126,6 +130,8 @@ mod iter {
     use super::*;
     use pretty_assertions::assert_eq;
 
+    #[allow(clippy::explicit_iter_loop)]
+    #[allow(clippy::never_loop)]
     #[test]
     fn test_iter_empty() {
         let vector: Vector<i32> = Vector::new();
@@ -135,6 +141,7 @@ mod iter {
         }
     }
 
+    #[allow(clippy::never_loop)]
     #[test]
     fn test_iter_empty_backwards() {
         let vector: Vector<i32> = Vector::new();
@@ -144,6 +151,7 @@ mod iter {
         }
     }
 
+    #[allow(clippy::explicit_iter_loop)]
     #[test]
     fn test_iter_big_vector() {
         let limit = 32 * 32 * 32 + 1;
@@ -379,7 +387,7 @@ fn test_macro_vector() {
     let vector_1 = Vector::new().push_back(1);
     let vector_1_2_3 = Vector::new().push_back(1).push_back(2).push_back(3);
 
-    assert_eq!(Vector::<u32>::new(), vector![]);
+    assert_eq!(Vector::<u32>::new(), vector![] as Vector<u32>);
     assert_eq!(vector_1, vector![1]);
     assert_eq!(vector_1_2_3, vector![1, 2, 3]);
 }
@@ -425,7 +433,7 @@ fn test_drop_last_drops_last_element() {
         vector = vector.drop_last().unwrap();
     }
 
-    assert_eq!(vector, Vector::new());
+    assert_eq!(vector, Vector::<i32>::new());
 }
 
 #[test]
@@ -619,7 +627,7 @@ fn test_partial_ord() {
     let vector_1_prime = vector!["a"];
     let vector_2 = vector!["b"];
     let vector_3 = vector![0.0];
-    let vector_4 = vector![core::f32::NAN];
+    let vector_4 = vector![f32::NAN];
 
     assert_eq!(vector_1.partial_cmp(&vector_1_prime), Some(Ordering::Equal));
     assert_eq!(vector_1.partial_cmp(&vector_2), Some(Ordering::Less));
@@ -719,10 +727,9 @@ fn test_index_mut() {
 #[cfg(feature = "serde")]
 #[test]
 fn test_serde() {
-    use bincode::{deserialize, serialize};
     let vector: Vector<i32> = vector![5, 6, 7, 8];
-    let encoded = serialize(&vector).unwrap();
-    let decoded: Vector<i32> = deserialize(&encoded).unwrap();
+    let encoded = serde_json::to_string(&vector).unwrap();
+    let decoded: Vector<i32> = serde_json::from_str(&encoded).unwrap();
 
     assert_eq!(vector, decoded);
 }
